Package: whatifbandit
Title: Analyzing Randomized Experiments as Multi-Arm Bandits
Version: 0.3.0
Authors@R: c(
    person("Noah", "Ochital", , "noahochital@icloud.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0009-0009-0398-2963")),
    person("Ryan T.", "Moore", , "rtm@american.edu", role = c("ctb", "cph"),
           comment = c(ORCID = "0000-0002-3916-8113"))
  )
Description: Simulates the results of completed randomized controlled
    trials, as if they had been conducted as adaptive Multi-Arm Bandit
    (MAB) trials instead. Augmented inverse probability weighted
    estimation (AIPW), outlined by Hadad et al. (2021)
    <doi:10.1073/pnas.2014602118>, is used to robustly estimate the
    probability of success for each treatment arm under the adaptive
    design. Provides customization options to simulate perfect/imperfect
    information, stationary/non-stationary bandits, blocked treatment
    assignments, along with control augmentation, and other hybrid
    strategies for assigning treatment arms. The methods used in
    simulation were inspired by Offer-Westort et al.  (2021)
    <doi:10.1111/ajps.12597>.
License: GPL (>= 3)
URL: https://github.com/Noch05/whatifbandit
BugReports: https://github.com/Noch05/whatifbandit/issues
Depends: R (>= 4.1.0)
Imports: bandit, data.table, dplyr, furrr, ggplot2, lubridate, purrr,
        randomizr, rlang, tibble, tidyr
Suggests: future, knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-10-29 16:28:47 UTC; noaho
Author: Noah Ochital [aut, cre, cph] (ORCID:
    <https://orcid.org/0009-0009-0398-2963>),
  Ryan T. Moore [ctb, cph] (ORCID:
    <https://orcid.org/0000-0002-3916-8113>)
Maintainer: Noah Ochital <noahochital@icloud.com>
Repository: CRAN
Date/Publication: 2025-11-03 10:20:02 UTC
Built: R 4.6.0; ; 2026-04-30 06:13:18 UTC; unix
