# Generated by roxygen2: do not edit by hand

S3method(plot,mab)
S3method(plot,multiple.mab)
S3method(print,mab)
S3method(print,multiple.mab)
S3method(summary,mab)
S3method(summary,multiple.mab)
export(multiple_mab_simulation)
export(single_mab_simulation)
importFrom(data.table,":=")
importFrom(data.table,.I)
importFrom(data.table,.N)
importFrom(rlang,"!!!")
importFrom(rlang,"!!")
importFrom(rlang,.data)
importFrom(stats,density)
